-- RS_DKP.lua
-- Part of RS_DKP by Tim Morgan
-- General and utility functions for RS_DKP.

-- DESCRIPTION
-- Resets all globals used by RS_DKP in preparation for a new raid

function omgp_initialize()
	omgp_currentRaidID = nil;
	omgp_bossDeathRecorded = {};
	omgp_lastIntervalUpdated = -1;
	omgp_instanceZones = nil;
	omgp_acceptableDeadZones = nil;
	omgp_resetBids();
	omgp_bossMobs = {};
	omgp_razorNotReallyDead = false;
	omgp_masterIsMe = true;
end

-- DESCRIPTION
-- Calls omgp_dkpCommandUsage if the command couldn't be handled by
-- omgp_handleDKPCommand
-- PARAMETERS
-- msg: the text following the "/dkp" command
-- RETURNS

function omgp_dkpCommand(msg)
	command = string.lower(msg);
	if (omgp_handleDKPCommand(command) == false) then omgp_dkpCommandUsage(); end
end

-- DESCRIPTION
-- Displays the /dkp command usage to the player

function omgp_dkpCommandUsage()
	omgp_pluginMessage("Start DKP with \"/dkp [instance]\", where [instance] is one of:", "NOTE");
	omgp_pluginMessage("  \"mc\", \"ony\", \"bwl\", \"aq40\", \"naxx\", or \"world\"", "NOTE");
	omgp_pluginMessage("  (Use this right when the raid begins.)", "NOTE");
	omgp_pluginMessage("End DKP with \"/dkp end\"; \"dkp master\" to take master.", "NOTE");
end

-- DESCRIPTION
-- Displays the /bids command usage to the player

function omgp_bidsCommandUsage()
	omgp_pluginMessage("/bids end - Ends the auction and announces the winner.", "NOTE");
	omgp_pluginMessage("/bids double - Ends a double auction, announcing the two winners.", "NOTE");
	omgp_pluginMessage("/bids cancel - Cancels the auction with no announcement.", "NOTE");
	omgp_pluginMessage("/bids check - Displays who the winner would be if the auction were to end. (Only displayed to you.)", "NOTE");
	omgp_pluginMessage("/bids chomp - Clears the highest bid. Use this if a whisper is mistaken for a bid.", "NOTE");
	omgp_pluginMessage("/bids call - Restates the current auction on the raid channel.", "NOTE");
end

-- DESCRIPTION
-- Attempts to handle a whisper, first using omgp_playerRequestedDKP, then
-- trying omgp_playerWithdrewBid, and finally omgp_playerPlacedBid. Does nothing
-- if none of those commands could handle the bid, or if DKP isn't currently
-- being recorded.
-- PARAMETERS
-- player: the name of the player that sent the message
-- msg: the message that was received

function omgp_whisperReceived(player, msg)
	if (omgp_currentRaidID == nil) then return end;
	
	if (omgp_playerRequestedDKP(player, msg) == true) then return; end
	if (player == UnitName("player")) then return; end
	if (omgp_playerWithdrewBid(player, msg) == true) then return; end
	omgp_playerPlacedBid(player, msg);
end

-- DESCRIPTION
-- Returns a list of the names of raiders who are currently in the raid and
-- eligible for DKP according to omgp_memberIsEligible.
-- PARAMETERS
-- includeEveryone: if true, ignores whether or not each member is eligible;
--                  just returns a list of members in the raid
-- RETURNS
-- The list of members, or an empty list if DKP isn't currently being recorded

function omgp_listOfActiveRaiders(includeEveryone)
	local memberList = {};
	local notEligible = {};
	local curMember;
	
	-- if omgp_currentRaidID == nil then return memberList; end
	
	for curMember = 1,40,1 do
		local memberName, memberRank, memberSubgroup, memberLevel, memberClass, memberClassCaps, memberZone, memberIsOnline, memberIsDead = GetRaidRosterInfo(curMember);
		local memberIsEligible, reasonWhyNot = omgp_memberIsEligible(memberName, memberZone, memberIsDead, memberIsOnline, curMember);
		
		if (
			memberName ~= nil -- if this is an actual player name
			and (
				memberIsEligible == true -- and either that person is active
				or includeEveryone == true -- or we don't care whether he is active
			)
		) then
			table.insert(memberList, memberName); -- add him to the list
		elseif (memberName ~= nil) then
			table.insert(notEligible, {memberName, reasonWhyNot}); -- otherwise add him to the ineligible list with an explanation as to why
		end
	end
	
	return memberList, notEligible;
end

-- DESCRIPTION
-- Returns true if a player has the ghost debuff (is dead and released).
-- PREREQS
-- The player must be dead, or this method may return incorrect information
-- PARAMETERS
-- memberNumber: the player's index in the raid list
-- RETURNS
-- True if the player has the ghost debuff, false if the player does not

function omgp_hasGhostDebuff(memberNumber)
	-- if we have rez sickness, the ghost debuff is in slot 2, otherwise in slot 1
	if ((UnitDebuff("raid" .. memberNumber, 1) ~= nil and string.find(UnitDebuff("raid" .. memberNumber, 1), "Ability_Vanish") ~= nil)
		or (UnitDebuff("raid" .. memberNumber, 2) ~= nil and string.find(UnitDebuff("raid" .. memberNumber, 2), "Ability_Vanish") ~= nil)) then
		return true;
	end
	return false;
end

-- DESCRIPTION
-- Displays a message to the user on his chat log. The message will be prefixed
-- with the "<RS_DKP>" indicator. Errors, warnings, and debug messages will
-- additionally be prefixed with a textual label indicating such.
-- PARAMETERS
-- level: "WARNING" for orange messages, "ERROR" for red messages, "DEBUG" for
--		  beige messages, any other value or ommitted for cyan messages

function omgp_pluginMessage(msg, level)
	if (level == "WARNING") then
		DEFAULT_CHAT_FRAME:AddMessage("<RS_DKP> WARNING: " .. msg, 1, 0.7, 0.5);
	elseif (level == "ERROR") then
		DEFAULT_CHAT_FRAME:AddMessage("<RS_DKP> ERROR: " .. msg, 1, 0.3, 0.3);
	elseif (level == "DEBUG") then
	DEFAULT_CHAT_FRAME:AddMessage("<RS_DKP> DEBUG: " .. msg, 1, 0.8, 0.3);
	else
		DEFAULT_CHAT_FRAME:AddMessage("<RS_DKP> " .. msg, 0.5, 1, 1);
	end
end

-- DESCRIPTION
-- Sends a message to the raid channel. The message will be prefixed with the
-- "<RS_DKP>" indicator.
-- PREREQS
-- The player must be in a raid group.
-- PARAMETERS
-- msg: the messag to send

function omgp_sendRaidMessage(msg)
	SendChatMessage("<RS_DKP> " .. msg, "RAID");
end

-- DESCRIPTION
-- Sends a message to the raid warning channel if the player is the raid leader;
-- otherwise sends it to the raid channel. The message will be prefixed with the
-- "<RS_DKP>" indicator.
-- PREREQS
-- The player must be in a raid group.
-- PARAMETERS
-- msg: the messag to send

function omgp_announceRaidMessage(msg)
	if (IsRaidLeader()) then SendChatMessage("<RS_DKP> " .. msg, "RAID_WARNING");
	else SendChatMessage("<RS_DKP> " .. msg, "RAID"); end
end

-- DESCRIPTION
-- Changes a textual representation of a number into a numerical value. This
-- only applies for strings that cannot be coerced into numbers the normal Lua
-- way, such as tokens returned from string.gfind.
-- PARAMETERS
-- numString: the string to convert
-- RETURNS
-- The number in the string. Returns 0 if no number was found. (Yeah, I know,
-- that's bad.)

function omgp_makeNumber(numString)
	-- Well, it turns out string.gfind returns tokens that somehow can't be
	-- converted into numbers, even if they are numbers, so we have to do the
	-- conversion ourselves
	
	numberSoFar = 0;
	local divisorCounter = 1;
	accumulationFunction = function(num, digit)
		return num*10 + digit
	end
	
	for digit in string.gfind(numString, "[%d%.]") do
		if (digit == "1") then
			numberSoFar = accumulationFunction(numberSoFar, 1);
		elseif (digit == "2") then
			numberSoFar = accumulationFunction(numberSoFar, 2);
		elseif (digit == "3") then
			numberSoFar = accumulationFunction(numberSoFar, 3);
		elseif (digit == "4") then
			numberSoFar = accumulationFunction(numberSoFar, 4);
		elseif (digit == "5") then
			numberSoFar = accumulationFunction(numberSoFar, 5);
		elseif (digit == "6") then
			numberSoFar = accumulationFunction(numberSoFar, 6);
		elseif (digit == "7") then
			numberSoFar = accumulationFunction(numberSoFar, 7);
		elseif (digit == "8") then
			numberSoFar = accumulationFunction(numberSoFar, 8);
		elseif (digit == "9") then
			numberSoFar = accumulationFunction(numberSoFar, 9);
		elseif (digit == "0") then
			numberSoFar = accumulationFunction(numberSoFar, 0);
		elseif (digit == ".") then
			accumulationFunction = function(num, digit)
				divisorCounter = divisorCounter * 10;
				return num + digit/divisorCounter;
			end
		end
	end
	return numberSoFar;
end

-- The following code strips outgoing whispers beginning with " <RS_DKP>". Note
-- that that's a space, followed by "<RS_DKP>". Messages that just begin with
-- "<RS_DKP>" are not stripped. That way, unimportant whispers like player DKP
-- reports are stripped, while bid confirmations are not.

-- Thanks to Darco for the idea & some of the code
omgp_OldChatFrame_OnEvent = ChatFrame_OnEvent;
function omgp_newOnEvent(event)
	if ( event == "CHAT_MSG_WHISPER_INFORM" ) then
		if ( string.find(arg1, " <RS_DKP>") or
			( PUBLIC_BIDDING == true and string.find(arg1, "<RS_DKP>")) ) then
			return;
		end
	end
	omgp_OldChatFrame_OnEvent(event);
end
ChatFrame_OnEvent = omgp_newOnEvent;

-- DESCRIPTION
-- Converts a string into a table of substrings separated by a given delimiter.
-- PARAMETERS
-- str: the string
-- pat: the delimiter, as a pattern match
-- RETURNS
-- A table of separated substrings, as ordered in the string

-- from http://lua-users.org/wiki/StringRecipes
function omgp_stringSplit(str, pat)
   local t = {n = 0}
   local fpat = "(.-)"..pat
   local last_end = 1
   local s,e,cap = string.find(str, fpat, 1)
   while s ~= nil do
      if s~=1 or cap~="" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s,e,cap = string.find(str, fpat, last_end)
   end
   if last_end<=string.len(str) then
      cap = string.sub(str,last_end)
      table.insert(t,cap)
   end
   return t
end

-- DESCRIPTION
-- Decodes an item identifier string into its name, color, and ID. Shamelessly
-- taken from Telo and the CT team.
-- PARAMETERS
-- sItem: the item identifier string (usually linked into chat from shift-click)
-- RETURNS
-- 1. the item's color
-- 2. the item's ID
-- 3. the item's name

function omgp_getItemInfo(sItem)
	-- Thanks to Telo for the following regular expression
	local iStart, iEnd, sColor, sItemName, sName = string.find(sItem, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h|r");
	if ( sColor and sItemName and sName ) then
		return sColor, sItemName, sName;
	end
end

-- DESCRIPTION
-- Handles system messages that could indicate someone joining or leaving the
-- raid group, among other things.
-- PARAMETERS
-- msg: the system message

function omgp_systemMessage(msg)
	-- someone joins the raid
	for token in string.gfind(msg, "(.+) has joined the raid group") do
		omgp_broadcastDKPInProgressInform(token);
		return;
	end
	for token in string.gfind(msg, "|Hplayer:(.+)|h%[(.+)%]|h has come online%.") do
		omgp_broadcastDKPInProgressInform(token);
		return;
	end
	
	-- you leave the raid
	if (msg == "You have left the raid group") then
		if (omgp_masterIsMe == true and omgp_currentRaidID ~= nil) then
			omgp_pluginMessage("You are the master and you have left the raid; stopped recording DKP for " .. omgp_currentRaidID .. ".", "WARNING");
			omgp_pluginMessage("You should tell someone else with the plug-in to run \"/dkp master\" so recording continues!", "WARNING");
			omgp_configureForEndingDKP();
		elseif (omgp_masterIsMe == false and omgp_currentRaidID ~= nil) then
			omgp_pluginMessage("Dropped sync for " .. omgp_currentRaidID .. "; you left the raid group.");
			omgp_configureForEndingDKP();
		elseif (omgp_masterIsMe == false and omgp_currentRaidId == nil) then
			omgp_masterIsMe = true;
		end
	end
end