-- RS_DKP_Bidding.lua
-- Part of RS_DKP by Tim Morgan
-- Functions and constants relating to taking bids and managing auctions.

--
-- CONSTANTS
--

local DKP_MIN_BID = 4 -- minimum number of DKP to win an item
local DKP_MIN_INTERVAL = 0.5 -- DKP cannot be bid in increments smaller than this

local PUBLIC_BIDDING = true -- are bids announced on raid chat?

local CLASS_PLURAL = {
	["Druid"] = "druids",
	["Paladin"] = "paladins",
	["Priest"] = "priests",
	["Mage"] = "mages",
	["Warlock"] = "warlocks",
	["Warrior"] = "warriors",
	["Shaman"] = "shaman",
	["Rogue"] = "rogues",
	["Hunter"] = "hunters"
};
local CLASSES_FOR_ABBREV = {
	["warrior"] = {["Warrior"] = true},
	["warriors"] = {["Warrior"] = true},
	["warr"] = {["Warrior"] = true},
	["warrs"] = {["Warrior"] = true},
	["druid"] = {["Druid"] = true},
	["druids"] = {["Druid"] = true},
	["drood"] = {["Druid"] = true},
	["droods"] = {["Druid"] = true},
	["dr00d"] = {["Druid"] = true},
	["dr00ds"] = {["Druid"] = true},
	["paladin"] = {["Paladin"] = true},
	["paladins"] = {["Paladin"] = true},
	["pally"] = {["Paladin"] = true},
	["pallys"] = {["Paladin"] = true},
	["pallies"] = {["Paladin"] = true},
	["priest"] = {["Priest"] = true},
	["priests"] = {["Priest"] = true},
	["rogue"] = {["Rogue"] = true},
	["rogues"] = {["Rogue"] = true},
	["rouge"] = {["Rogue"] = true},
	["rouges"] = {["Rogue"] = true},
	["warlock"] = {["Warlock"] = true},
	["warlocks"] = {["Warlock"] = true},
	["lock"] = {["Warlock"] = true},
	["locks"] = {["Warlock"] = true},
	["shaman"] = {["Shaman"] = true},
	["shamans"] = {["Shaman"] = true},
	["shammy"] = {["Shaman"] = true},
	["shammys"] = {["Shaman"] = true},
	["shammies"] = {["Shaman"] = true},
	["hunter"] = {["Hunter"] = true},
	["hunters"] = {["Hunter"] = true},
	["mage"] = {["Mage"] = true},
	["mages"] = {["Mage"] = true},
	["vendor"] = {["Mage"] = true},
	["vendors"] = {["Mage"] = true},
	
	["melee"] = {["Rogue"] = true, ["Warrior"] = true},
	["melees"] = {["Rogue"] = true, ["Warrior"] = true},
	["meleers"] = {["Rogue"] = true, ["Warrior"] = true},
	["healer"] = {["Priest"] = true, ["Druid"] = true, ["Shaman"] = true, ["Paladin"] = true},
	["healers"] = {["Priest"] = true, ["Druid"] = true, ["Shaman"] = true, ["Paladin"] = true},
	["caster"] = {["Mage"] = true, ["Warlock"] = true},
	["casters"] = {["Mage"] = true, ["Warlock"] = true},
	
	["cloth"] = {["Mage"] = true, ["Warlock"] = true, ["Priest"] = true},
	["leather"] = {["Druid"] = true, ["Rogue"] = true},
	["mail"] = {["Hunter"] = true, ["Shaman"] = true},
	["chain"] = {["Hunter"] = true, ["Shaman"] = true},
	["plate"] = {["Warrior"] = true, ["Paladin"] = true}
};

--
-- FUNCTIONS
--

-- DESCRIPTION
-- Handles bids commands. Displays command usage if the command could not be
-- recognized. If the command is a valid linked item, begins bidding on that
-- item using omgp_startBidding.
-- PARAMETERS
-- msg: the text following "/bids "

function omgp_bidsCommand(msg)
	if (omgp_currentRaidID == nil) then
		omgp_pluginMessage("You aren't currently recording DKP.", "ERROR");
		return;
	end
	if (CT_RaidTracker_GetItemInfo == nil) then
		omgp_pluginMessage("Could not interface with CT_RaidTracker. Perhaps you installed some crazy new version I don't support?", "ERROR");
		return;
	end
	
	local itemColor, itemID, itemName = omgp_getItemInfo(msg);
	
	if (omgp_itemUpForBids == nil) then
		omgp_startBidding(msg, itemName);
	else
		if (msg == nil) then
			omgp_pluginMessage("You are currently holding an auction.", "NOTE");
			omgp_bidsCommandUsage();
			return;
		end
	
		if (string.lower(msg) == "end") then
			omgp_endBidding(false);
		elseif (string.lower(msg) == "double") then
			omgp_endBidding(true);
		elseif (string.lower(msg) == "cancel") then
			omgp_cancelBidding();
		elseif (string.lower(msg) == "chomp") then
			omgp_clearBid(nil, false);
		elseif (string.lower(msg) == "check") then
			omgp_reportBids();
		elseif (string.lower(msg) == "call") then
			omgp_announceRaidMessage("Reminder: Whisper your bids to me for " .. omgp_itemUpForBids .. "!");
		elseif (itemName ~= null) then
			omgp_pluginMessage("You are already taking bids on " .. omgp_itemUpForBids .. ". Please end the current auction first.", "ERROR");
		else
			local aMatch = false;
			for token in string.gfind(string.lower(msg), "chomp (%a+)$") do
				omgp_clearBid(token, false);
				aMatch = true;
				break;
			end
			if (aMatch == false) then
				omgp_pluginMessage("I didn't recognize that command. Here is a list of commands:", "NOTE");
				omgp_bidsCommandUsage();
			end
		end
	end
end

-- DESCRIPTION
-- Begins an auction on a linked item. Only players that are a part of the raid
-- at the time this method is called will be eligible to bid.
-- PREREQS
-- Relevant globals must have been reset with omgp_resetBids 
-- PARAMETERS
-- item: a linked item
-- itemName: the linked item's textual name

function omgp_startBidding(item, itemName)
	if (item == nil) then
		omgp_pluginMessage("To start a bid, type \"/bids [Item]\" (link in the item by shift-clicking).", "NOTE");
		return;
	end
	
	if (itemName == nil) then
		 omgp_pluginMessage("Item not recognized. Type \"/bids [Item]\" to start an auction (link in the item by shift-clicking).", "NOTE");
	else
		omgp_itemUpForBids = item;
		omgp_eligibleBidders = {};
		local member;
		local eligibles = omgp_listOfActiveRaiders(true);
		for member in eligibles do
			omgp_eligibleBidders[eligibles[member]] = true;
		end
		omgp_announceRaidMessage("Whisper your bids to me for " .. item .. "!");
		--if (omgp_playerDKP ~= nil) then
		--	local myName = UnitName("player");
		--	omgp_sendRaidMessage("/w " .. myName .. " dkp - Find out your DKP");
		--	omgp_sendRaidMessage("/w " .. myName .. " dkp Someplayer - Find out someone else's DKP");
		--	omgp_sendRaidMessage("/w " .. myName .. " dkp someclass - A list of DKP by class");
		--end
	end
end

-- DESCRIPTION
-- Ends the current auction and announces the winner if possible. If a tie
-- currently exists, announces the tie and continues bidding with a new
-- eligibility list and minimum bid.
-- PREREQS
-- CT_RaidTracker validity must have previously been checked in
-- omgp_startBidding, and an auction must be currently in progress.

function omgp_endBidding(doubleBid)
	if (omgp_hasDoneCheckBeforeEnd ~= true and PUBLIC_BIDDING == false) then
		omgp_pluginMessage("You should do a \"/bids check\" to make sure bids are correct before ending the auction.", "ERROR");
		return;
	end
	
	local highestBidders, highestBid = omgp_getHighestBid(1);
	if (table.getn(highestBidders) == 0) then
		omgp_sendRaidMessage("There were no bids for the last auction.");
		omgp_resetBids();
		return;
	elseif (table.getn(highestBidders) == 2 and doubleBid == true) then
		omgp_sendRaidMessage(highestBidders[1] .. " wins with a bid of " .. highestBid .. " DKP!");
		omgp_sendRaidMessage("And " .. highestBidders[2] .. " wins with a bid of " .. highestBid .. " DKP!");
		local itemColor, itemID, itemName, itemTooltip = CT_RaidTracker_GetItemInfo(omgp_itemUpForBids);
		omgp_logWinningBid(itemName, highestBidders[1], highestBid);
		omgp_logWinningBid(itemName, highestBidders[2], highestBid);
		omgp_resetBids();
		return;
	elseif (table.getn(highestBidders) == 1) then
		omgp_sendRaidMessage(highestBidders[1] .. " wins with a bid of " .. highestBid .. " DKP!");
		local itemColor, itemID, itemName, itemTooltip = CT_RaidTracker_GetItemInfo(omgp_itemUpForBids);
		omgp_logWinningBid(itemName, highestBidders[1], highestBid);
	else
		omgp_minimumBid = highestBid;
		omgp_bids = {};
		omgp_eligibleBidders = {};
		local member;
		for member in highestBidders do
			omgp_eligibleBidders[highestBidders[member]] = true;
		end
		omgp_sendRaidMessage("There has been a tie! The following people should rebid with a minimum bid of " .. highestBid .. " DKP: " .. table.concat(highestBidders, ", "));
		return;
	end
	
	if (doubleBid == false) then
		omgp_resetBids();
		return;
	end
	
	highestBidders, highestBid = omgp_getHighestBid(2);
	if (table.getn(highestBidders) == 0) then
		omgp_sendRaidMessage("Nobody claimed the second item.");
	elseif (table.getn(highestBidders) == 1) then
		omgp_sendRaidMessage("And " .. highestBidders[1] .. " wins with a bid of " .. highestBid .. " DKP!");
		local itemColor, itemID, itemName, itemTooltip = CT_RaidTracker_GetItemInfo(omgp_itemUpForBids);
		omgp_logWinningBid(itemName, highestBidders[1], highestBid);
		omgp_resetBids();
	else
		omgp_minimumBid = highestBid;
		omgp_bids = {};
		omgp_eligibleBidders = {};
		local member;
		for member in highestBidders do
			omgp_eligibleBidders[highestBidders[member]] = true;
		end
		omgp_sendRaidMessage("There has been a tie for the second item! The following people should rebid with a minimum bid of " .. highestBid .. " DKP: " .. table.concat(highestBidders, ", "));
		return;
	end
	
	omgp_resetBids();
end

-- DESCRIPTION
-- Cancels the current bid without recording a winner.
-- PREREQS
-- An auction must be currently in progress.

function omgp_cancelBidding()
	omgp_sendRaidMessage("The auction has been canceled!");
	omgp_resetBids();
end

-- DESCRIPTION
-- Clears the highest bid. If more than one player is tied for the highest bid
-- and no player is specified through the "name" parameter, reports to the
-- player that more information is required. If the name parameter is specified,
-- clears the bid of the given player.
-- PREREQS
-- An auction must be in progress.
-- PARAMETERS
-- name: the name of the player to clear; if ommitted, clears the highest bidder
-- searchAllBids: if true, searches all bids for the player's bid; otherwise,
--				  searches only the high bid(s)
-- RETURNS
-- True if a bid was cleared, false if the bid to clear could not be found

function omgp_clearBid(name, searchAllBids)
	local highBidders, highBid = omgp_getHighestBid(1);
	
	if (table.getn(highBidders) == 0) then
		omgp_pluginMessage("There have been no bids yet.", "ERROR");
	elseif (table.getn(highBidders) > 1 and name == nil) then
		omgp_pluginMessage("The high bid of " .. highBid .. " is held by the following people: " .. table.concat(highBidders, ", ") .. ".", "ERROR");
		omgp_pluginMessage("Rerun this command as \"/bids chomp Playername\" to clear the correct player's high bid.", "ERROR");
	else
		foundPlayer = false;
		for n in omgp_bids do
			if (searchAllBids == false and omgp_bids[n][2] < highBid) then
				break;
			end
			if (name == nil or string.upper(omgp_bids[n][1]) == string.upper(name)) then
				local oldPlayer = omgp_bids[n][1];
				local oldBid = omgp_bids[n][2];
				table.remove(omgp_bids, n);
				table.sort(omgp_bids, function(a, b) return a[2] > b[2] end);
				
				omgp_pluginMessage("Deleted " .. oldPlayer .. "'s bid of " .. oldBid .. " DKP.", "NOTE");
				if (PUBLIC_BIDDING == true) then
					local unused, newHighBid = omgp_getHighestBid(1);
					if (newHighBid == -1) then
						omgp_sendRaidMessage("A bid was withdrawn, and now there are no more bids.");
					elseif (newHighBid < highBid) then omgp_sendRaidMessage("The high bid has dropped to " .. newHighBid .. " DKP."); end
				end
				
				foundPlayer = true;				
				break;
			end
		end
		if (foundPlayer == false) then
			omgp_pluginMessage("Couldn't find player '" .. name .. "'.", "ERROR");
		end
	end
	return foundPlayer;
end

-- DESCRIPTION
-- Displays a list of current bids on the chat log.

function omgp_reportBids()
	if (table.getn(omgp_bids) == 0) then
		omgp_pluginMessage("No bids so far.", "NOTE");
	else
		for n in omgp_bids do
			omgp_pluginMessage(omgp_bids[n][1] .. ": " .. omgp_bids[n][2] .. " DKP", "NOTE");
		end
	end
	omgp_hasDoneCheckBeforeEnd = true;
end

-- DESCRIPTION
-- Returns the highest bid and bidder(s) so far in the current auction.
-- PREREQS
-- An auction must be currently in progress.
-- RETURNS
-- 1. A list of the bidders holding the high bid
-- 2. The highest bid

function omgp_getHighestBid(place)
	-- if no one bid, we're done already
	if (table.getn(omgp_bids) == 0) then
		return {}, -1;
	end
	
	local currentHigh = -1;
	local currentPlace = 0;
	local highestBid = -1; -- will store the nth highest bid
	local highestBidders = {}; -- will store the nth highest bidders
	for index,bid in ipairs(omgp_bids) do
		if (bid[2] < currentHigh or currentHigh == -1) then
			currentHigh = bid[2];
			currentPlace = currentPlace + 1;
		end
		if (currentPlace == place) then
			highestBid = bid[2];
			table.insert(highestBidders, bid[1]);
		end
		if (bid[2] < highestBid) then
			break;
		end
	end
	
	return highestBidders, highestBid;
end

-- DESCRIPTION
-- Resets all globals used by the bidding system. Any ongoing auctions will be
-- canceled without warning or notification.

function omgp_resetBids()
	omgp_itemUpForBids = nil;
	omgp_minimumBid = DKP_MIN_BID;
	omgp_bids = {};
	omgp_eligibleBidders = {};
	omgp_hasDoneCheckBeforeEnd = false;
end

-- DESCRIPTION
-- Reports the DKP of raid members of a given class to the given player. The
-- list will be whispered to the player, sorted in order of DKP, highest to
-- lowest.
-- PREREQS
-- Must be in a raid group and omgp_playerDKP must be loaded and initialized.
-- PARAMETERS
-- givenClassName: the class or class group
-- player: the player to whisper
-- RETURNS
-- True if information was reported; false if the class name was invalid

function omgp_reportClassDKPToPlayer(givenClassName, player)
	local classes = CLASSES_FOR_ABBREV[string.lower(givenClassName)];
	if (classes == nil) then return false; end
	
	local dkpList = {};
	for curMember = 1,40,1 do
		local memberName, unused1, memberSubgroup, unused2, memberClass = GetRaidRosterInfo(curMember);
		if (memberName ~= nil) then
			local playerDKP = omgp_playerDKP[string.upper(memberName)];
			if (classes[memberClass] == true and playerDKP ~= nil) then
				table.insert(dkpList, {
					["name"] = memberName,
					["dkp"] = playerDKP
				});
			end
		end
	end
	
	if (table.getn(dkpList) == 0) then
		SendChatMessage(" <RS_DKP> No one of that class (or those classes) in this raid group have any DKP.", "WHISPER", nil, player);
	else
		table.sort(dkpList, function(a, b) return a["dkp"] > b["dkp"] end);
		for curMember in dkpList do
			SendChatMessage(" <RS_DKP> " .. dkpList[curMember]["name"] .. ": " .. dkpList[curMember]["dkp"] .. " DKP", "WHISPER", nil, player);
		end
	end
	
	return true;
end

-- DESCRIPTION
-- Reports a player's DKP to another (possibly the same) player.
-- PREREQS
-- omgp_playerDKP must be loaded and initialized.
-- PARAMETERS
-- playersDKP: the player whose DKP is to be reported
-- playerToTell: the player to whisper
-- RETURNS
-- True if information was reported; false if playersDKP was not found in the
-- player list

function omgp_reportDKPToPlayer(playersDKP, playerToTell)
	local dkp = omgp_playerDKP[string.upper(playersDKP)];
	if (dkp == nil) then
		return false;
	else
		SendChatMessage(" <RS_DKP> That player has " .. dkp .. " DKP.", "WHISPER", nil, playerToTell);
		return true;
	end
end

-- DESCRIPTION
-- Investigates the possibility that an incoming whisper was a withdraw request,
-- and if so, withdraws the player's bid.
-- PREREQS
-- An auction must be in progress.
-- PARAMETERS
-- player: the player who whispered
-- msg: the message whispered
-- RETURNS
-- True if a bid was withdrawn; false if the whisper was not a withdraw request

function omgp_playerWithdrewBid(player, msg)
	if (omgp_itemUpForBids == nil or string.lower(msg) ~= "withdraw") then return false; end
	
	if (omgp_clearBid(player, true)) then
		SendChatMessage("<RS_DKP> Your bid has been withdrawn.", "WHISPER", nil, player);
	else
		SendChatMessage("<RS_DKP> You haven't placed a bid.", "WHISPER", nil, player);
	end
	
	return true; -- we cleared a player's bid
end

-- DESCRIPTION
-- Investigates the possibility that an incoming whisper was a bid, and if so,
-- records the player's bid.
-- PREREQS
-- An auction must be in progress.
-- PARAMETERS
-- player: the player who whispered
-- msg: the message whispered
-- RETURNS
-- True if a bid was recorded; false if the whisper was not a bid

function omgp_playerPlacedBid(player, msg)
	if (omgp_itemUpForBids == nil) then return false; end
	
	local tokens = {}; -- will contain all the numbers found within the string
	
	-- Bids can be of the form "4", "4.5", or ".5", and we need to match all those possibilities
	for num in string.gfind(msg, "%.?%d+%.?%d*") do
		-- determine if the first character is a period, and there's another period beyond that
		if (string.sub(num, 1, 1) == "." and string.find(msg, "%.", 2) ~= nil) then
			-- do nothing; i just wanted to word the above conditional in the positive
		else
			table.insert(tokens, num);
		end
	end
	
	if (table.getn(tokens) > 1) then
		SendChatMessage("<RS_DKP> If that was a bid, I couldn't understand it because it had more than one number in it.", "WHISPER", nil, player);
	elseif (table.getn(tokens) == 1) then
		theBid = omgp_makeNumber(tokens[1]); -- token strings cannot be coerced into numbers
		omgp_recordBid(player, theBid);
		return true;
	end
end

-- DESCRIPTION
-- Attempts to record a player's bid. Reports back to the player if the bid was
-- not valid or the player was not eligible. Otherwise, records the bid.
-- PREREQS
-- An auction must be in progress.
-- PARAMETERS
-- player: the player who bidded
-- bid: the amount of the bid

function omgp_recordBid(player, bid)
	-- First the error checking
	
	if (omgp_eligibleBidders[player] ~= true) then
		SendChatMessage("<RS_DKP> You are not eligible to bid on this item.", "WHISPER", nil, player);
		return;
	end
	if (omgp_playerDKP[string.upper(player)] ~= nil and omgp_playerDKP[string.upper(player)] < bid) then
		SendChatMessage("<RS_DKP> You only have " .. omgp_playerDKP[string.upper(player)] .. " DKP. Your bid was rejected.", "WHISPER", nil, player);
		return;
	end
	if (bid < omgp_minimumBid) then
		SendChatMessage("<RS_DKP> The minimum bid is " .. omgp_minimumBid .. " DKP. Your bid was rejected.", "WHISPER", nil, player);
		return;
	end
	if (mod(bid, DKP_MIN_INTERVAL) ~= 0) then
		SendChatMessage("<RS_DKP> Bids must be in increments of " .. DKP_MIN_INTERVAL .. ". Your bid was rejected.", "WHISPER", nil, player);
		return;
	end
	
	-- OK, so it's a valid bid. Let's record it
	
	local unused, oldHighBid = omgp_getHighestBid(1);
	
	-- Has the player previously placed a bid? If so, we replace the old bid
	local foundPlayer = false;
	for n in omgp_bids do
		if (omgp_bids[n][1] == player) then
			foundPlayer = omgp_bids[n][2]; -- foundPlayer stores the player if a prev bid was placed, or false if no prev bid was found
			omgp_bids[n][2] = bid;
			break;
		end
	end
	-- If no prev bid was made, record the new bid
	if (foundPlayer == false) then
		table.insert(omgp_bids, {player, theBid});
	end
	
	-- Confirm to the player that the bid was recorded
	SendChatMessage("<RS_DKP> Your bid of " .. bid .. " DKP has been recorded.", "WHISPER", nil, player);
	
	-- Keep the bids table sorted for quick access to the high bidder
	table.sort(omgp_bids, function(a, b) return a[2] > b[2] end);
	
	-- If we're reporting bids publicly, do so
	if (PUBLIC_BIDDING == true) then
		local unused2, highBid = omgp_getHighestBid(1);
		if (highBid > oldHighBid) then omgp_sendRaidMessage("The high bid is now " .. highBid .. " DKP."); end
		if (bid == highBid and highBid == oldHighBid) then omgp_sendRaidMessage("Another high bid of " .. highBid .. " DKP has been placed."); end
		if (highBid < oldHighBid) then omgp_sendRaidMessage("The high bid has dropped to " .. highBid .. " DKP."); end
	end
	
	-- We need to do another /bids check if we end the bidding at this point
	omgp_hasDoneCheckBeforeEnd = false;
end