-- RS_DKP_Recording.lua
-- Part of RS_DKP by Tim Morgan
-- Functions for the recording of raid events and the reporting of DKP.

--
-- CONSTANTS
--

local EVENT_POKE = "POKE" -- periodic, regular check of raid attendance
local EVENT_LOOT = "LOOT" -- loot acquired through DKP bid
local EVENT_BOSS = "BOSS" -- a boss kill
local EVENT_ATTEMPT = "ATTEMPT" -- a boss attempt
local EVENT_INSTANCE = "INST" -- new instance record
local EVENT_ONTIME = "ONTIME" -- ontime bonus

local DKP_ACCUMULATION_INTERVAL = 300 -- number of seconds between DKP eligibility checks

local IMPORTANT_BOSSES = {
	["Molten Core"] = {
		["Lucifron"] = true,
		["Magmadar"] = true,
		["Gehennas"] = true,
		["Garr"] = true,
		["Baron Geddon"] = true,
		["Shazzrah"] = true,
		["Sulfuron Harbinger"] = true,
		["Golemagg the Incinerator"] = true,
		["Ragnaros"] = true
	},
	["Onyxia's Lair"] = {
		["Onyxia"] = true
	},
	["Blackwing Lair"] = {
		["Razorgore the Untamed"] = true,
		["Vaelastrasz the Corrupt"] = true,
		["Broodlord Lashlayer"] = true,
		["Chromaggus"] = true,
		["Ebonroc"] = true,
		["Firemaw"] = true,
		["Flamegor"] = true,
		["Nefarian"] = true
	},
	["Temple of Ahn'Qiraj"] = {
		["The Prophet Skeram"] = true,
		["Battleguard Sartura"] = true,
		["Fankriss the Unyielding"] = true,
		["Princess Huhuran"] = true,
		["Emperor Vek'nilash"] = true,
		["C'Thun"] = true,
		-- all three bugs die: Lord Kri, Vem, Princess Yauj
		["Viscidus"] = true,
		["Ouro the Sandworm"] = true
	},
	["Naxxramas"] = {
		["Anub'Rekhan"] = true,
		["Grand Widow Faerlina"] = true,
		["Maexxna"] = true,
		["Patchwerk"] = true,
		["Grobbulus"] = true,
		["Gluth"] = true,
		["Thaddius"] = true,
		--["Feugen"] = true,
		--["Stalagg"] = true,
		["Noth the Plaguebringer"] = true,
		["Heigan the Unclean"] = true,
		["Loatheb"] = true,
		["Instructor Razuvious"] = true,
		["Gothik the Harvester"] = true,
		["Sapphiron"] = true,
		["Kel'Thuzad"] = true
		-- all four horseman have to die: Highlord Morgraine, Thane Korthazz, Sir Zelliek, Lady Blameaux
	},
	["world"] = {
		["Lord Kazzak"] = true,
		["Azuregos"] = true,
		["Taerar"] = true,
		["Lethon"] = true,
		["Emeriss"] = true,
		["Ysondre"] = true
	}
};
local INSTANCE_ZONES = {
	["mc"] = {
		["Molten Core"] = true,
		["The Molten Core"] = true,
		["Magmadar Cavern"] = true,
		["Ragnaros' Lair"] = true
	},
	["ony"] = {
		["Onyxia's Lair"] = true
	},
	["bwl"] = {
		["Blackwing Lair"] = true,
		["Dragonmaw Garrison"] = true,
		["Shadow Wing Lair"] = true
	},
	["aq40"] = {
		["Ahn'Qiraj"] = true
	},
	["naxx"] = {
		["Naxxramas"] = true
	}
};
local DEAD_ZONES = {
	["mc"] = {
		["Thorium Point"] = true,
		["Firewatch Ridge"] = true,
		["Searing Gorge"] = true,
		["Blackrock Mountain"] = true,
		["The Molten Span"] = true,
		["Blackrock Spire"] = true,
		["Molten Core"] = true,
		["The Cauldron"] = true
	},
	["ony"] = {
		["Brackenwall Village"] = true,
		["Dustwallow Marsh"] = true,
		["The Quagmire"] = true,
		["The Dragonmurk"] = true,
		["Stonemaul Ruins"] = true,
		["Wyrmbog"] = true,
		["Onyxia's Lair"] = true
	},
	["bwl"] = {
		["Thorium Point"] = true,
		["Firewatch Ridge"] = true,
		["Searing Gorge"] = true,
		["Blackrock Mountain"] = true,
		["The Molten Span"] = true,
		["Blackrock Spire"] = true,
		["Blackwing Lair"] = true,
		["The Cauldron"] = true
	},
	["aq40"] = {
		["Silithus"] = true,
		["The Scarab Wall"] = true,
		["Gates of Ahn'Qiraj"] = true
	},
	["naxx"] = {
		["Eastern Plaguelands"] = true,
		["Naxxramas"] = true
	}
};
local INSTANCE_NAME = {
	["mc"] = "Molten Core",
	["ony"] = "Onyxia's Lair",
	["bwl"] = "Blackwing Lair",
	["aq40"] = "Temple of Ahn'Qiraj",
	["naxx"] = "Naxxramas",
	["world"] = "world"
};

--
-- FUNCTIONS
--

-- DESCRIPTION
-- Determines if a DKP command is a start or stop request, and handles it
-- appropriately.
-- PARAMETERS
-- command: the text following "/dkp ", lowercase-normalized
-- RETURNS
-- True if the command was handled, false if it wasn't recognized

function omgp_handleDKPCommand(command)
	if (INSTANCE_ZONES[command] ~= nil and DEAD_ZONES[command] ~= nil or command == "world") then
		omgp_startDKPRecording(command);
		return true;
	elseif (command == "off" or command == "stop" or command == "end") then
		omgp_stopDKPRecording();
		return true;
	elseif (command == "master") then
		omgp_takeMaster();
		return true;
	end
	
	return false;
end

-- DESCRIPTION
-- Loads the list of eligible boss kills for a DKP run.
-- PARAMETERS
-- instance: the name of the instance being raided

function omgp_initializeBosses(instance)
	omgp_bossMobs = IMPORTANT_BOSSES[instance];
	if (omgp_bossMobs == nil) then omgp_bossMobs = {}; end
end

-- DESCRIPTION
-- Begins recording DKP for a raid. Sets self as master and begins recording
-- DKP, logging the event and ontime bonus.
-- PARAMETERS
-- command: the /dkp command instance indicator

function omgp_startDKPRecording(command)
	if (omgp_currentRaidID ~= nil) then
		omgp_pluginMessage("Already recording DKP for raid ID " .. omgp_currentRaidID .. ".", "ERROR");
		return;
	end
	if (UnitInRaid("player") ~= 1) then
		omgp_pluginMessage("You must be a part of a raid group to record DKP.", "ERROR");
		return;
	end
	if (omgp_masterIsMe == false) then
		omgp_pluginMessage("Someone else is already recording DKP for this raid, and you are not part of the linkup.", "ERROR");
		return;
	end
	
	omgp_configureForBeginningDKP(command);
	omgp_pluginMessage("Recording DKP for new raid ID " .. omgp_currentRaidID .. "...", "NOTE");
	
	local eventID = omgp_logEvent(EVENT_INSTANCE, { ["instance"]=INSTANCE_NAME[command] });
	omgp_broadcastMessage(eventID, "INST", {command, omgp_currentRaidID});
	omgp_masterIsMe = true;
	
	omgp_updateInterval();
	omgp_logRaidAttendanceEvent(EVENT_ONTIME);
end

-- DESCRIPTION
-- Called when another raid member begins recording DKP. Begins recording DKP,
-- setting self as slave.
-- PARAMETERS
-- eventID: the ID to use for the INST event
-- command: the /dkp command instance indicator
-- originator: the name of the player that began recording DKP

function omgp_remoteDKPBegin(eventID, command, raidID, originator)
	if (omgp_currentRaidID ~= nil) then
		omgp_pluginMessage(originator .. " began recording DKP for " .. INSTANCE_NAME[command] .. ", but I am already recording DKP.", "ERROR");
		return;
	end
	if (UnitInRaid("player") ~= 1) then
		omgp_pluginMessage(originator .. " began recording DKP for " .. INSTANCE_NAME[command] .. ", but you are no longer in a raid.", "ERROR");
		return;
	end
	
	omgp_configureForBeginningDKP(command, raidID);
	
	local eventIDNum = omgp_logEvent(EVENT_INSTANCE, { ["instance"]=INSTANCE_NAME[command], ["remote"]=originator }, eventID);
	omgp_masterIsMe = false;
	omgp_pluginMessage("Syncing with " .. originator .. ", who started new raid ID " .. omgp_currentRaidID .. "...", "NOTE");
	omgp_broadcastMessage(eventIDNum, "INSTACK");
end

-- DESCRIPTION
-- Sets up global variables for a new DKP record.
-- PARAMETERS
-- command: the /dkp command instance indicator
-- raidID: if given, assigns a given raid ID, as opposed to an auto-generated one

function omgp_configureForBeginningDKP(command, raidID)
	omgp_initialize();
	omgp_initializeBosses(INSTANCE_NAME[command]);
	omgp_instanceZones = INSTANCE_ZONES[command];
	omgp_acceptableDeadZones = DEAD_ZONES[command];
	omgp_resetBids();
	
	if (raidID == nil) then
		omgp_currentRaidID = date("%Y%m%d%H%M%S") .. "/" .. INSTANCE_NAME[command];
	else
		omgp_currentRaidID = raidID;
	end
end

-- DESCRIPTION
-- Ends recording for a DKP run. Resets appropriate globals.

function omgp_stopDKPRecording()
	if (omgp_currentRaidID == nil) then
		omgp_pluginMessage("DKP is not currently being recorded.", "NOTE");
		return;
	end
	if (omgp_masterIsMe == false) then
		omgp_pluginMessage("You're not the master; only the master can end the raid.", "ERROR");
		return;
	end
	
	omgp_pluginMessage("DKP recording ended for raid ID " .. omgp_currentRaidID .. ". Happy trails!", "NOTE");
	omgp_broadcastMessage(0, "END", {omgp_currentRaidID});
	omgp_configureForEndingDKP();
end

function omgp_remoteDKPEnd(sender)
	if (omgp_currentRaidID == nil) then
		omgp_masterIsMe = true; -- in case we we got nabbed with an INFORM
		return;
	end
	if (omgp_masterIsMe == true) then
		omgp_pluginMessage(sender .. " claims that he's the master, and so do you. Ignoring his END request.", "ERROR");
		return;
	end
	
	omgp_pluginMessage(sender .. " ended DKP for raid ID " .. omgp_currentRaidID .. "; link dropped. Happy trails!", "NOTE");
	omgp_configureForEndingDKP();
end

function omgp_configureForEndingDKP()
	omgp_currentRaidID = nil;
	omgp_masterIsMe = true;
	omgp_bossDeathRecorded = {};
	omgp_instanceZones = nil;
	omgp_acceptableDeadZones = nil;
	omgp_resetBids();
	omgp_initializeBosses("N/A");
end

-- DESCRIPTION
-- Meant to be periodically called; polls and records raid attendance using
-- omgp_logRaidAttendanceEvent as appropriate. The method firsts determines if
-- either not enough time or too much time has passed between now and the
-- previous invocation of this method. If not enough time has passed, this
-- method exits as no new polling is required. If too much time has passed, this
-- method back-polls current eligible players an appropriate amount of times to
-- cover the skipped interval(s).
-- PREREQS
-- Must be in a raid group.

function omgp_periodicPoke()
	if (omgp_currentRaidID == nil) then return; end
	local timesWeMissed = omgp_updateInterval();
	if (omgp_masterIsMe == false) then return; end
	
	if (timesWeMissed == 0) then
		omgp_logRaidAttendanceEvent(EVENT_POKE);
	elseif (timesWeMissed > 0) then
		omgp_pluginMessage("No update during the last interval! Back-awarding DKP for the missed intervals with the current list of eligible people...", "WARNING");
		-- We'll go back and, for each interval we missed, give everyone DKP
		for unused = 1,timesWeMissed,1 do
			omgp_logRaidAttendanceEvent(EVENT_POKE);
		end
		-- Then add the DKP for this interval, which we can do accurately
		omgp_logRaidAttendanceEvent(EVENT_POKE);
	end
end

function omgp_updateInterval()
	local currentInterval = math.floor(GetTime()/DKP_ACCUMULATION_INTERVAL);
	local prevIntervalUpdated = omgp_lastIntervalUpdated;
	if (omgp_lastIntervalUpdated < currentInterval) then
		omgp_lastIntervalUpdated = currentInterval;
	end
	
	if (prevIntervalUpdated + 1 == currentInterval) then
		return 0;
	elseif (prevIntervalUpdated + 1 < currentInterval) then
		return (currentInterval-prevIntervalUpdated-1);
	end
	return -1; -- no need to update at all
end

-- DESCRIPTION
-- Polls players for DKP eligibility and records the event to the log.
-- PREREQS
-- Must be in a raid group.
-- PARAMETERS
-- eventToUse: The event type to log as, usually EVENT_POKE or EVENT_ONTIME

function omgp_logRaidAttendanceEvent(eventToUse)
	if (omgp_currentRaidID == nil) then return end;
	
	local memberList, notEligibles = omgp_listOfActiveRaiders(false);
	local eventID = omgp_logEvent(eventToUse, { ["members"] = memberList, ["not_eligible"] = notEligibles });
	
	omgp_broadcastMessage(eventID, "POLL", {eventToUse});
	omgp_broadcastMyPoll(eventID);
end

-- DESCRIPTION
-- Broadcasts the people who earned attendance credit for the given poll event,
-- so that slaves add these people to their event.
-- PREREQS
-- eventID must be a valid POKE or ONTIME event.
-- PARAMETERS
-- eventID: the ID of the event

function omgp_broadcastMyPoll(eventID)
	for index,player in pairs(omgp_getEventData(eventID, "members")) do
		omgp_broadcastMessage(eventID, "CREDIT", {player});
	end
end

-- DESCRIPTION
-- Logs a raid attendance event with a given ID, then broadcasts its findings to
-- others as a series of POKED events.
-- PREREQS
-- Must be in a raid group.
-- PARAMETERS
-- eventID: the ID to use for the event
-- eventToUse: The event type to log as, usually EVENT_POKE or EVENT_ONTIME
-- eventType: the event type to log as

function omgp_remoteActivatedPoll(eventID, eventToUse, sender)
	local memberList = omgp_listOfActiveRaiders(false);
	for index,player in ipairs(memberList) do
		omgp_broadcastMessage(eventID, "POKED", {player});
	end
	omgp_logEvent(eventToUse, { ["members"] = {} }, eventID);
end

-- DESCRIPTION
-- Called when another player polls a raid member; adds the raid member to our
-- list if necessary.
-- PREREQS
-- Must have the event of ID eventID created.
-- PARAMETERS
-- eventID: the ID of the event
-- playerName: the player that another user polled as available

function omgp_playerPolledRemotely(eventID, playerName)
	if (omgp_masterIsMe == false) then return; end
	
	for index,player in ipairs(omgp_getEventData(eventID, "members")) do
		if (playerName == player) then return; end
	end
	
	-- wasn't in the list, let's add him
	table.insert(omgp_getEventData(eventID, "members"), playerName);
	omgp_broadcastMessage(eventID, "CREDIT", {playerName});
end

-- DESCRIPTION
-- Sent by the master to inform slaves that a given player gets credit for an
-- attendance event.
-- PREREQS
-- Must have the event of ID eventID created.
-- PARAMETERS
-- eventID: the ID of the event
-- playerName: the player that will receive credit

function omgp_addPlayerCredit(eventID, playerName)
	if (omgp_currentRaidID == nil) then return; end
	if (omgp_masterIsMe == true) then
		omgp_pluginMessage("Someone else sent a CREDIT request, but you're the master. Ignoring his CREDIT request.", "ERROR");
		return;
	end
	
	table.insert(omgp_getEventData(eventID, "members"), playerName);
end

-- DESCRIPTION
-- Returns true if a player is eligible for DKP. A player is eligible if he
-- is online, in a valid zone (or dead and released in a valid zone), and not
-- AFK. If a player is ineligible, additionally returns the reason why not.
-- PREREQS
-- In a raid group, CT_RaidAssist installed
-- PARAMETERS
-- memberName: the name of the player
-- memberZone: the name of the player's current zone
-- memberIsDead: whether or not the player is dead
-- memberIsOnline: whether or not the player is online
-- memberNumber: the player's index in the raid list
-- RETURNS
-- 1. True if the player is eligible for DKP, false if not
-- 2. A human-readable reason why the player was not eligible for DKP, if
--    applicable

function omgp_memberIsEligible(memberName, memberZone, memberIsDead, memberIsOnline, memberNumber)
	if (memberName == nil) then return false; end
	
	if (memberIsOnline == false or memberZone == "Offline") then
		return false, "Offline";
	end
	
	if (omgp_acceptableDeadZones ~= nil
		and omgp_hasGhostDebuff(memberNumber) == true
		and omgp_acceptableDeadZones[memberZone] ~= true) then
		if (memberZone == nil) then memberZone = "[nil]"; end
		return false, "Released and in zone " .. memberZone;
	end
	
	if (CT_RA_Stats == nil) then
		omgp_pluginMessage("Couldn't interface with CT_RaidAssist. Perhaps you installed some crazy new version I'm not compatible with?", "ERROR");
		return false, "Couldn't link to CTRA!";
	end
	
	if (CT_RA_Stats[memberName] ~= nil and CT_RA_Stats[memberName]["AFK"]) then
		return false, "AFK";
	end
	
	if (omgp_instanceZones ~= nil
		and memberZone ~= nil
		and omgp_instanceZones[memberZone] ~= true
		and omgp_hasGhostDebuff(memberNumber) == false) then
		
		if (memberZone == nil) then memberZone = "[nil]"; end
		return false, "In zone " .. memberZone;
	end
	
	return true;
end

-- DESCRIPTION
-- Catches and responds to boss /yelled messages of interest. For instance,
-- Majordomo doesn't actually die; he yells a certain message when he is
-- defeated. Some messages indicate "deaths," others indicate boss attempts
-- (such as Razorgore). Not all boss attempts can be recorded.
-- PARAMETERS
-- msg: the message yelled

function omgp_possibleEmoteDeath(msg)
	if (omgp_currentRaidID == nil) then return end;
	
	-- EMOTE DEATHS
	
	-- Domo (doesn't actually die)
	if (string.find(msg, "Brashly, you have come to wrest") ~= null) then
		omgp_handleBossKill("Majordomo Executus");
	end
	-- Razorgore (dies every attempt, even failed attempts)
	if (string.find(msg, "If I fall into the abyss") ~= null or
		string.find(msg, "That device shall never") ~= null) then
		omgp_razorNotReallyDead = true;
		debugprofilestart();
	end
	
	-- EMOTE ATTEMPTS
	
	-- Razorgore
	if (string.find(msg, "Protect the eggs at all costs") ~= null) then
		local memberList = omgp_listOfActiveRaiders(false);
		if (memberList == nil) then memberList = ""; end;
		omgp_logEvent(EVENT_ATTEMPT, { ["boss"] = "Razorgore the Untamed", ["members"] = memberList } );
	end
end

-- DESCRIPTION
-- Records a boss kill if this computer is the master, otherwise requests that a
-- boss kill be recorded.
-- PARAMETERS
-- bossName: the name of the boss

function omgp_handleBossKill(bossName, eventID)
	if (omgp_bossDeathRecorded[bossName] == true) then return; end
	if (omgp_currentRaidID == nil) then return; end
	if (omgp_masterIsMe == false and eventID == nil) then
		omgp_broadcastMessage(0, "SBOSS", {bossName});
		return;
	end
	
	omgp_bossDeathRecorded[bossName] = true;
	local memberList = omgp_listOfActiveRaiders(false);
	if (memberList == nil) then memberList = ""; end;
	if (eventID == nil) then
		local eventID = omgp_logEvent(EVENT_BOSS, { ["boss"] = bossName, ["members"] = memberList } );
		omgp_broadcastMessage(eventID, "BOSS", {bossName});
	else
		omgp_logEvent(EVENT_BOSS, { ["boss"] = bossName, ["members"] = memberList }, eventID);
	end
end

-- DESCRIPTION
-- Called when a combat log message is received; checks if the message is a
-- death message and records the boss death if necessary. Also handles the
-- special case of Razorgore, who dies every attempt regardless of success.
-- PARAMETERS
-- msg: the combat log message

function omgp_mobDies(msg)
	if (omgp_currentRaidID == nil) then return end;
	
	local mobName;
	for mobName in string.gfind(msg, "(.+) dies%.") do
		-- this shouldn't be called more than once
		if (omgp_bossMobs[mobName] == true) then
			-- check for the razorgore special case
			if (mobName == "Razorgore the Untamed"
				and omgp_razorNotReallyDead == true) then
				
				local elapsedTime = debugprofilestop();
				if (elapsedTime < 10) then
					omgp_razorNotReallyDead = false;
					return;
				end
				-- basically we assume that if 10 seconds have passed, we
				-- missed his death last time around, and this one is for real
			end
			
			omgp_handleBossKill(mobName);
		end
	end
end

-- DESCRIPTION
-- Logs a winning bid. Also deducts that amount from the player's current DKP.
-- If the computer is a slave, not a master, broadcasts a SLOOT event so the
-- master can record the item won.
-- PREREQS
-- DKP recording must be underway.
-- PARAMETERS
-- item: the name of the item won
-- winner: the name of the winner
-- bid: the amount of DKP to deduct
-- eventID: optional parameter, specifies the position in the events table
-- RETURNS
-- The ID of the new LOOT event, or -1 if the event was not logged because this
-- computer is a slave

function omgp_logWinningBid(item, winner, bid, eventID)
	if (omgp_masterIsMe == false and eventID == nil) then
		omgp_broadcastMessage(0, "SLOOT", {item, winner, bid});
		return -1;
	end
	if (omgp_currentRaidID == nil) then return; end
	
	local eventIDAssigned;
	if (eventID == nil) then
		eventIDAssigned = omgp_logEvent(EVENT_LOOT, { ["winner"] = winner, ["item"] = item, ["bid"] = bid });
	else
		eventIDAssigned = omgp_logEvent(EVENT_LOOT, { ["winner"] = winner, ["item"] = item, ["bid"] = bid }, eventID);
	end
	
	if (omgp_playerDKP[string.upper(winner)] ~= nil) then
		omgp_playerDKP[string.upper(winner)] = omgp_playerDKP[string.upper(winner)] - bid;
	end
	
	if (eventID == nil) then
		omgp_broadcastMessage(eventIDAssigned, "LOOT", {item, winner, bid});
	end
end

-- DESCRIPTION
-- Logs an event and associated information.
-- PREREQS
-- DKP recording must be underway.
-- PARAMETERS
-- eventType: the event type (see constants)
-- eventData: additional information associated with the event, as a hash with
--            string keys
-- eventID: optional parameter, the ID of the event; chooses the next available
--          ID if not provided
-- RETURNS
-- the ID of the new event

function omgp_logEvent(eventType, eventData, eventID)
	local eventTime = date("%Y%m%d%H%M%S");
	if (eventData == nil) then eventData = {}; end
	--debugmsg(eventType .. " (" .. eventTime .. "): " .. eventData);
	
	if (omgp_currentRaidID == nil) then return end;
	
	-- If a log for this raid has not been started, start one
	if (omgp_raidHistory[omgp_currentRaidID] == nil) then
		omgp_raidHistory[omgp_currentRaidID] = {};
	end
	
	-- Get the new location to insert this latest log event
	-- omgp_raidHistory is a dictionary of tables by raid ID; the hashed table a
	-- list of events stored as a table, THAT table is { time, event, data }
	if (eventID == nil) then
		table.insert(omgp_raidHistory[omgp_currentRaidID], { ["time"]=eventTime, ["event"]=eventType, ["data"]=eventData });
		return table.getn(omgp_raidHistory[omgp_currentRaidID]);
	else
		omgp_raidHistory[omgp_currentRaidID][eventID] = { ["time"]=eventTime, ["event"]=eventType, ["data"]=eventData };
		return eventID;
	end
end

-- DESCRIPTION
-- Returns the user-information "data" fields of an event. Returns nil if DKP is
-- not currently being recorded.
-- PARAMETERS
-- eventID: the ID of the event
-- RETURNS
-- the userdata associated with the new event

function omgp_getEventData(eventID, dataKey)
		if (omgp_currentRaidID == nil) then return nil; end
		return omgp_raidHistory[omgp_currentRaidID][eventID]["data"][dataKey];
end

-- Investigates the possibility that an incoming whisper was a DKP request, and
-- handles it if appopriate. Handles all three request types: just "dkp",
-- "dkp [player]", and "dkp [class or group]".
-- PREREQS
-- Must be in a raid group and recording DKP.
-- PARAMETERS
-- player: the player that whispered
-- msg: the whispered message
-- RETURNS
-- True if DKP was reported to the player, false if the message is not a DKP
-- request

function omgp_playerRequestedDKP(player, msg)
	-- Handle just "dkp"
	if (string.lower(msg) == "dkp") then
		if (omgp_reportDKPToPlayer(player, player) == false) then
			SendChatMessage(" <RS_DKP> You aren't a part of the DKP system yet.", "WHISPER", nil, player);
		end
		return true; -- we reported a player's DKP
	end
	
	-- Handle "dkp [something]"
	if (string.sub(string.lower(msg), 1, 3) ~= "dkp") then return false; end -- bug in Lua implementation means we can't constrain our pattern to the start of the string
	for token in string.gfind(string.lower(msg), "dkp (%a+)$") do
		local tokenWasValidClass = omgp_reportClassDKPToPlayer(token, player);
		if (tokenWasValidClass == false) then
			local tokenWasValidName = omgp_reportDKPToPlayer(token, player);
			if (tokenWasValidName == false) then
				SendChatMessage(" <RS_DKP> Could not find player '" .. token .. "'. (Did you spell it correctly?) Maybe that person isn't in the DKP system.", "WHISPER", nil, player);
				return true; -- we reported someone else's DKP to the player
			end
		else
			return true; -- we reported class DKP to the player
		end
	end
	
	return false;
end